import typing as t
from urge import procs

# from .base import Action, action
# from .procs import weather, youdao, browser, memegen, fileoperation
from urge.base import Action, action
from urge.procs import weather, youdao, browser, memegen, fileoperation
from dataclasses import dataclass
from path import Path


class do(Action):
    '''
    Do, do my work\n
    Do my dirty work, scapegoat\n
    '''

    procs = []

    def __init__(self, *args) -> None:
        assert isinstance(self.procs, list)
        for f in args:
            self.procs.append(f)


@dataclass
class web_screenshot(Action):
    url: str
    procs = [browser.web_screenshot]


@dataclass
class get_now_temp(Action):
    city: str
    ascii_graphs: bool = True
    procs = [weather.get_weather, weather.ret_weather]
    # Should add something like this
    # procs = [weather.get_weather, _p(weather.ret_weather,{'ascii_graphs':True})]
    # procs = [weather.get_weather, _p(weather.ret_weather,ascii_graphs=True)]
    # The idea here is turn {"ascii_graphs":True} into self.ascii_graphs=True


@dataclass
class get_simple_temp(Action):
    city: str
    ascii_graphs: bool = False
    procs = [weather.get_weather, weather.ret_weather]


@dataclass
class translate(Action):
    q: str
    full: bool = False
    procs = [youdao.trans_post, youdao.trans_filter]


@dataclass
class simple_translate(Action):
    q: str
    procs = [youdao.simple_translate]


@dataclass
class easy_ocr(Action):
    path: str
    procs = [youdao.pic2base64, youdao.ocr_post, youdao.ocr_ret]


@dataclass
class pic_translate(Action):
    path: str
    procs = [youdao.pic2base64, youdao.pictrans_post, youdao.pictrans_write]


@dataclass
class handwrite_ocr(Action):
    path: str
    procs = [youdao.pic2base64, youdao.ocr_post, youdao.ocr_ret]


@dataclass
class receipt_ocr(Action):
    path: str
    procs = [youdao.pic2base64, youdao.receipt_ocr_post, youdao.receipt_ocr_ret]


@dataclass
class table_ocr(Action):
    path: str
    procs = [youdao.pic2base64, youdao.table_ocr_post, youdao.table_write_excel]


@dataclass
class meme_gen(Action):
    name: str
    up: t.Optional[str]
    down: str
    procs = [memegen.get_meme]


@dataclass
class list_all(Action):
    path: str
    procs = [fileoperation.list_all]


@dataclass
class create(Action):
    path: str
    procs = [fileoperation.create]


@dataclass
class create_folder(Action):
    path: str
    procs = [fileoperation.create_folder]


class find:
    def __init__(self, path: t.Text, pattern: t.Text, recursive=False) -> None:
        self.handler = fileoperation.walk if recursive else fileoperation.find
        self._list = self.handler(path=path, pattern=pattern)
        if isinstance(self._list, t.Generator):
            self._list = list(self._list)
            # when self.__dict__ deepcopy it goes nuts
            # Maybe I can do self.pipe(seed) without deepcopy...

    @action([fileoperation._delete_all])
    def delete(self):
        return dict(f_list=self._list)

    @action([fileoperation._rename_all])
    def rename(self, with_suffix: str = None, with_preafix: str = None):
        return locals()

    @action([fileoperation._zip_files])
    def zip(self, with_folder=False):
        return dict(f_list=self._list, with_folder=with_folder)

    @action([fileoperation])
    def unzip(self):
        ...

    @action([fileoperation._move_all])
    def move(self, dst: str):
        return dict(f_list=self._list, dst=dst)



from path import Path
from fs.copy import copy_fs
from fs.zipfs import ZipFS
from fs.osfs import OSFS
import zipfile as z
import typing as t


# =============================== File finders functions ===============================
# Tested it works
def walk(path: str, pattern: str):
    return Path(path).walk(pattern)


# Tested it works
def list_all(path: str):
    # clean up.
    return Path(path).listdir()


# Tested ,It works
def find(path: str, pattern: str, **kwd):
    return Path(path).listdir(pattern)


# =============================== CRUD(files&dirs) functions ===============================
# ╭─────────────────────────────────────────────╮
# │ >>>>>>> Create(files&dirs) functions <<<<<< │
# ╰─────────────────────────────────────────────╯
# Tested it works
def create(path: str, **kwd):
    # Considering when dir not exist
    return Path(path).touch()


def create_folder(path: str, **kwd):
    p = Path(path)
    p.mkdir()


# ╭─────────────────────────────────────────────╮
# │ >>>>>>> Delete(files&dirs) functions <<<<<< │
# ╰─────────────────────────────────────────────╯

# Tested it works(Almost perfect)
def smart_delete(path: str):
    p = Path(path)
    if p.isdir():
        p.rmtree()
    elif p.isfile():
        p.remove()


# Tested it works(But feel... a litte weird?)
def delete(path: str, **kwd):
    p = Path(path)
    if p.isdir():
        # must be an exist dir
        print(
            f"The '{p}' is a dir(folder), and will not be deleted, if you want to"
            " delete a folder try to use delete_folder"
        )
        return
    p.remove()


def rmdir(path: str):
    p = Path(path)
    p.rmtree_p()


def rmdir_all(f_list: t.List[Path]):
    for p in f_list:
        rmdir(p)


# Tested ,it works
def _delete_all(f_list: list):
    for f in f_list:
        smart_delete(f)


# ╭─────────────────────────────────────────────╮
# │ >>>>>>> Rename(files&dirs) functions <<<<<< │
# ╰─────────────────────────────────────────────╯
#  Status:It works(Not done yet)
def rename(
    path: str, new_name: str = None, with_suffix: str = None, with_prefix: str = None
):
    if new_name:
        if with_suffix or with_prefix:
            raise ValueError(
                "You don't need to do that, considering in this"
                " way: 'prefix-newname.suffix'"
            )
        suffix = Path(path).ext
        _new_name = Path(path).dirname() / new_name + suffix
    elif with_suffix and with_prefix:
        # if new_name exists... wow...
        _new_name = Path(path).dirname() / with_prefix + Path(path).stem + with_suffix
    elif with_suffix:
        if new_name:
            raise ValueError("You can just pass 'newname.suffix' to new_name ")
        _new_name = Path(path).with_suffix(with_suffix)
    elif with_prefix:
        if new_name:
            raise ValueError("You can just pass 'prefix-newname.suffix' to new_name ")
        _new_name = Path(path).dirname() / with_prefix + Path(path).name

    else:
        raise ValueError("Need to pass at least one argument")

    Path(path).rename(_new_name)


def _rename_all(f_list: t.List[Path], with_prefix: str = None, with_suffix: str = None):

    if not with_prefix and not with_prefix:
        raise ValueError("To change files name, Must have a prefix or suffix ")

    for f in f_list:
        rename(f, with_suffix=with_suffix, with_prefix=with_prefix)


# ╭─────────────────────────────────────────────╮
# │ >>>>>>> Delete(files&dirs) functions <<<<<< │
# ╰─────────────────────────────────────────────╯
# Fixed, then test it.
def move(src: str, dst: str):
    p = Path(src)
    p.move(dst)


def _move_all(f_list: t.List[Path], dst: str):
    for f in f_list:
        f.move(dst)


# =============================== Zip archieves functions ===============================
# ╭─────────────────────────────────────────────╮
# │ >>>>>>> OldZip(files&dirs) functions <<<<<< │
# ╰─────────────────────────────────────────────╯
# Tested ,it works(first appear in lesson 13)
def zip(src: str, dst: str):
    src, dst = Path(src), Path(dst)
    ffs = copy_fs(OSFS(dst), ZipFS(src))

    return ffs


def _zip_all(f_list: t.List[Path]):
    for f in f_list:
        ...


def unzip(src: str, dst: str):
    src, dst = Path(src), Path(dst)
    ffs = copy_fs(ZipFS(src), OSFS(dst, create=True))
    return ffs


def _upzip_all(f_list: t.List[Path]):
    for f in f_list:
        unzip(f, f.stem)


# ╭─────────────────────────────────────────────╮
# │ >>>>>>> NewZip(files&dirs) functions <<<<<< │
# ╰─────────────────────────────────────────────╯
# Not tested yet(replace the old)
exclude = [".DS_Store", "__MACOSX", "__pycache__"]


def zip_files(folder_path: str, /, without_dir=False):
    if not Path(folder_path).isdir():
        raise ValueError(f"{folder_path} is not a dir(folder)")

    # could make it better here, but not this time
    # count = len(same_name)
    # name = f"{name}({count})"

    dir_name = Path(folder_path).abspath().basename()
    ac_path = Path(folder_path).abspath() / dir_name + ".zip"
    files = [f for f in Path(folder_path).files() if not f.basename() in exclude]

    with z.ZipFile(ac_path, compression=z.ZIP_DEFLATED, mode='w') as zp:
        if without_dir:
            for f in files:
                zp.write(f.abspath(), f.name)
            return

        for f in files:
            zp.write(f)


def _zip_files(f_list: t.List[Path], with_folder=False, **kwd):

    first = f_list[0]
    default_tar = first.parent / "archive.zip"

    with z.ZipFile(default_tar, compression=z.ZIP_DEFLATED, mode='w') as zp:
        if with_folder:
            for f in f_list:
                zp.write(f.abspath(), f.parent.basename() / f.name)
                # Filename:file location  Arcname:file struct in zipfile
            return
        for f in f_list:
            zp.write(f.abspath(), f.name)


def zipit(f_path: str):
    assert Path(f_path).isfile()

    parent_dir, file_name = (Path(f_path).abspath().parent, Path(f_path).abspath().stem)
    ac_path = parent_dir / file_name + ".zip"
    with z.ZipFile(ac_path, compression=z.ZIP_DEFLATED, mode="w") as zp:
        zp.write(Path(f_path), Path(f_path).name)


def unzipit(ac_path: str, target: t.Union[Path, str] = Path("./")):

    import shutil

    shutil.unpack_archive(ac_path, target)


# =============================== FS utils functions ===============================


def empty_check(f_list: t.List[Path], **kwd):

    if not f_list:
        raise ValueError("No files found")

    return locals()


def hidden_check(f_list: t.List[Path], **kwd):
    return dict(f_list=[f for f in f_list if not f.basename() in exclude]).update(kwd)



import time
import copy
import typing as t
import schedule as sche
from functools import partial, wraps

# from .utils import convert_time, callable_check, drop
from urge.utils import convert_time, callable_check, drop


class Action:

    # procs: t.Sequence[t.Callable]
    procs: t.Union[t.List, "ProcManager"]

    def once(self):
        ret = self._excute()
        return ret

    def every(self, freq) -> None:
        # shoud be in convert_time ?
        if isinstance(freq, dict):
            freq = convert_time(freq)

        sche.every(freq).seconds.do(self.once)

    def when(self, time: str):
        sche.every().day.at(time).do(self.once)

    def _excute(self):
        if not self.procs:
            raise Exception()
        elif isinstance(self.procs, list):
            self.procs = ProcManager(*self.procs)

        # seed = copy.deepcopy(self.__dict__)
        seed = self.__dict__.copy()
        seed = drop(seed, ["procs", "self"])
        ret = self.procs.excute(seed)
        # ret = self.procs(seed)
        # ret = self.procs(self.__dict__)
        # DONE Need change procs() -> procs.excute() -> way better
        return ret

    @classmethod
    def set_procs(cls, procs: t.List):
        cls.procs = procs
        return cls()


class ProcManager:
    def __init__(self, *procs) -> None:
        map(callable_check, procs)
        # self._procs += procs -> Because instances share their class _proc
        self._procs = procs

    def __add__(self, other):
        _new_procs = self._procs + other._procs
        return ProcManager(*_new_procs)

    def __call__(self, seed):
        ret = self.pipe(seed, *self._procs)
        return ret

    def excute(self, seed):
        ret = self.pipe(seed, *self._procs)
        return ret

    def pipe(self, seed, *funcs) -> t.Any:

        # TODO
        # when is a Action instance

        # _funcs = arg_load()

        for func in funcs:
            if isinstance(seed, dict):
                seed = func(**seed)
                continue
            seed = func(seed)
        return seed


class action(Action):
    def __init__(self, procs: t.List) -> None:
        self.procs = procs

    def __call__(self, decorated):
        # TODO really need a smarter way to detect self, and get result from procs
        @wraps(decorated)
        def instead(*args, **kwd) -> Action:
            seed = decorated(*args, **kwd)
            self.__dict__.update(seed)
            return self

        return instead


def require():
    '''
    load some .py file from a path as module to run
    this fuction returns Action
    '''
    pass


def env(**kwd):
    import os

    os.environ.update(kwd)


def start():
    while True:
        time.sleep(1)
        sche.run_pending()



from urge.base import ProcManager, action
from path import Path


def web_screenshot(url: str, target: str = "./", **kwargs) -> str:
    from playwright.sync_api import sync_playwright
    from datetime import datetime

    print("🤖 : Chromium import is OK...")

    with sync_playwright() as p:
        iphone_11 = p.devices["iPhone 11 Pro"]
        browser_type = p.chromium
        print("🤖 : Hey, I found the chromium binary...")
        print(browser_type)
        print(
            '🤖 : Launching browser... Please wait for a while then you will see the'
            ' result.'
        )
        browser = browser_type.launch(headless=True)
        context = browser.new_context(**iphone_11)
        page = context.new_page()
        page.set_viewport_size({"width": 375, "height": 635})
        page.goto(url, wait_until="networkidle")
        # page.screenshot(path=f'./example-{browser_type.name}haha.png')
        target = Path(target) 
        file_name = target/f'screenshot{datetime.now().strftime("%m-%d-%H-%M-%S")}.png'
        page.screenshot(path=file_name)
        print(f"🤖 : DONE")
        print(f'The file is at {file_name}')
        browser.close()
    return file_name


def get_screenshot(url):
    p = ProcManager(web_screenshot)
    return p(url)
